@@ -4,7 +4,7 @@ package JSON::Schema::Modern::Result;
 # vim: set ts=8 sts=2 sw=2 tw=100 et :
 # ABSTRACT: Contains the result of a JSON Schema evaluation
 
-our $VERSION = '0.589';
+our $VERSION = '0.590';
 
 use 5.020;
 use Moo;
@@ -92,7 +92,7 @@ has recommended_response => (
 );
 
 # strict_basic can only be used with draft2019-09.
-use constant OUTPUT_FORMATS => [qw(flag basic strict_basic detailed verbose terse data_only)];
+use constant OUTPUT_FORMATS => [qw(flag basic strict_basic terse data_only)];
 
 has output_format => (
   is => 'rw',
@@ -192,6 +192,7 @@ sub format ($self, $style, $formatted_annotations = undef) {
     return join("\n", uniq(map $_->stringify, $self->errors));
   }
 
+  # TODO: support detailed, verbose ?
   die 'unsupported output format';
 }
 
@@ -262,7 +263,7 @@ JSON::Schema::Modern::Result - Contains the result of a JSON Schema evaluation
 
 =head1 VERSION
 
-version 0.589
+version 0.590
 
 =head1 SYNOPSIS
 
@@ -316,7 +317,7 @@ Returns an array of L<JSON::Schema::Modern::Annotation> objects.
 
 =for stopwords subschemas
 
-One of: C<flag>, C<basic>, C<strict_basic>, C<detailed>, C<verbose>, C<terse>, C<data_only>. Defaults to C<basic>.
+One of: C<flag>, C<basic>, C<strict_basic>, C<terse>, C<data_only>. Defaults to C<basic>.
 
 =over 4
 
